# Copyright 2020-2024 Intel Corporation
# SPDX-License-Identifier: Apache-2.0


from openfl.interface.aggregation_functions.adagrad_adaptive_aggregation import (
    AdagradAdaptiveAggregation,
)
from openfl.interface.aggregation_functions.adam_adaptive_aggregation import AdamAdaptiveAggregation
from openfl.interface.aggregation_functions.core import AggregationFunction
from openfl.interface.aggregation_functions.fed_bagging import FedBaggingXGBoost
from openfl.interface.aggregation_functions.fedcurv_weighted_average import FedCurvWeightedAverage
from openfl.interface.aggregation_functions.geometric_median import GeometricMedian
from openfl.interface.aggregation_functions.median import Median
from openfl.interface.aggregation_functions.secure_weighted_average import SecureWeightedAverage
from openfl.interface.aggregation_functions.weighted_average import WeightedAverage
from openfl.interface.aggregation_functions.yogi_adaptive_aggregation import YogiAdaptiveAggregation
