# Copyright 2020-2024 Intel Corporation
# SPDX-License-Identifier: Apache-2.0


from openfl.component.aggregator.aggregator import Aggregator
from openfl.component.aggregator.straggler_handling import (
    CutoffTimePolicy,
    PercentagePolicy,
    StragglerPolicy,
    WaitForAllPolicy,
)
